/**************************************************************************************
SHERICAL SUDOKU PROJECT - MAIN JAVASCRIPT LIBRARY FILE
Version       : 1.0
Programmer(s) : Fahri Aydos ( sudoku@aydos.net / www.aydos.net )
First Version : 2006 05 08
Last Version  : 2006 05 08
---------------------------------------------------------------------------------------
Spherical Sudoku Copyright 2006 by Fahri Aydos
http://sudoku.aydos.net - spherical sudoku official website
---------------------------------------------------------------------------------------
This file is freeware...
You may copy, modify, use and distribute this software,
provided that you do not change this header and include it in all such copies.
**************************************************************************************/

/**************************************************************************************
SHERICAL SUDOKU GAMES AND THEIR PROGRAMMERS
1. Spherical         : Fahri Aydos
2. All Free          : Fahri Aydos
3. Mosaic            : Fahri Aydos
4. Fantasy           : Fahri Aydos
5. ...
6. ...
**************************************************************************************/

/**************************************************************************************
VERSION HISTORY
version 1.0, 2006 05 08
-- Table design
-- Spherical rotate functions
-- Static cells
-- Check table for sudoku rules
-- Game file design
-- Default games (Spherical,  All Free, Mosaic, Fantasy)
**************************************************************************************/

// VARIABLES ------------------------------------------------------------------------

// gamestat is for game status
//  0 : not started or finished;
//  1 : just begined, cells are rotating randomly
//  2 : started, player can use buttons
var gamestat=0;

// sudoku: main varible which hold the numbers on the table...
// at the beginning it satisfies sudoku rules
var sudoku = new Array();
sudoku[1] = new Array(0,9,6,3,1,7,4,2,5,8);
sudoku[2] = new Array(0,1,7,8,3,2,5,6,4,9);
sudoku[3] = new Array(0,2,5,4,6,8,9,7,3,1);
sudoku[4] = new Array(0,8,2,1,4,3,7,5,9,6);
sudoku[5] = new Array(0,4,9,6,8,5,2,3,1,7);
sudoku[6] = new Array(0,7,3,5,9,6,1,8,2,4);
sudoku[7] = new Array(0,5,8,9,7,1,3,4,6,2);
sudoku[8] = new Array(0,3,1,7,2,4,6,9,8,5);
sudoku[9] = new Array(0,6,4,2,5,9,8,1,7,3);
// temporary sudoku to clear board,
// this varibale does not changed during the game,
// so we can fill the main sudoku variable with this, to clear the table and to begin a new game
var tsudoku = new Array;
var i, j;
for (i=1;i<=9;i++) {
	tsudoku[i] = new Array;
	for (j=1;j<=9;j++) {
		tsudoku[i][j] = sudoku[i][j];
	}
}

// rows and coloms are easy to locate; what about the square!!!
// squares is to locate this small 3x3 squares, and check if it satisfies sudoku rules
var squares = new Array();
squares[1] = new Array(0,11,12,13,21,22,23,31,32,33);
squares[2] = new Array(0,14,15,16,24,25,26,34,35,36);
squares[3] = new Array(0,17,18,19,27,28,29,37,38,39);
squares[4] = new Array(0,41,42,43,51,52,53,61,62,63);
squares[5] = new Array(0,44,45,46,54,55,56,64,65,66);
squares[6] = new Array(0,47,48,49,57,58,59,67,68,69);
squares[7] = new Array(0,71,72,73,81,82,83,91,92,93);
squares[8] = new Array(0,74,75,76,84,85,86,94,95,96);
squares[9] = new Array(0,77,78,79,87,88,89,97,98,99);

// statics is for ramdomly choosen STATIC cells,
// used only in game: spherical
var statics = new Array();
statics[1] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[2] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[3] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[4] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[5] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[6] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[7] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[8] = new Array(0,0,0,0,0,0,0,0,0,0);
statics[9] = new Array(0,0,0,0,0,0,0,0,0,0);

/*	colors
	These are the colors of cells; We have three conditions:
		1. Cell's row has contain each number once
		2. Cell's colom has contain each number once
		3. Cell's small square has contain each number once
	If cell satisfies all of these conditions then it's color is "col3"
	If it satisfies any two conditions then it's color is "col2"
	If it satisfies only one condition then it's color is "col1"
	If it not satisfy any conditions  then it's color is "col0"
	We need "rgb0", "rgb1", "rgb2" variables which are equivalent to "col0", "col1", "col2"
	to check "object.style.backgroundColor" property in Firefox
*/
var colstatic = "#ff8080"; // rgb(255, 128, 128); red
var col0 = "#ffffff"; // rgb(255, 255, 255); white
var col1 = "#cccccc"; // rgb(204, 204, 204); light gray
var col2 = "#999999"; // rgb(153, 153, 153); dark gray
var col3 = "#99ccff"; // rgb(153, 204, 255); blue
var rgb0 = "rgb(255, 255, 255)";
var rgb1 = "rgb(204, 204, 204)";
var rgb2 = "rgb(153, 153, 153)";

// END OF VARIABLES ------------------------------------------------------------------------

function emailme() { // Copyright 1996 Jon Eyrick
	who="sudoku";
	where="aydos.net";
	parent.location.href='mailto:'+who+'@'+where;
}

/*	INIT_TABLE
	fill the sudoku table with INIT data
*/
function init_table() {
	var i, j;
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			sudoku[i][j] = tsudoku[i][j];
		}
	}
	fill_table();
	init_statics();
}

/*	RESET_TABLE
	to finish the game and reset the table to satisfy sudoku rules
*/
function reset_table() {
	gamestat=0;
	init_table();
	return false;
}

/*	FILL_TABLE
	fill the sudoku table with CURRENT data
*/
function fill_table() {
	var i, j, b, t;
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			b = document.getElementById('box'+i+j);
			b.value = sudoku[i][j];
			if(statics[i][j]==1){
				b.style.backgroundColor = colstatic;
			} else {
				b.style.backgroundColor = col0;
			}
		}
	}
	if(gamestat!=1)
		check_table();
}

/*	CHECK_TABLE
	chack if cells and table satisfy the sudoku rules
*/
function check_table() {
	var i, j, k, b, c, ok;
	var a = new Array();
	var c = 0; //count of cells which satisfies all sudoku rules
	/*
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			b = document.getElementById('box'+i+j);
			b.style.backgroundColor = col0;
		}
	}*/
	// rows
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			a[j] = 0;
		}
		ok = 1;
		for (j=1;j<=9;j++) {
			k = sudoku[i][j];
			if(a[k]==0) a[k]=k; else ok=0;
		}
		if(ok==1){
			for (j=1;j<=9;j++) {
				b = document.getElementById('box'+i+j);
				if((b.style.backgroundColor==col0)||(b.style.backgroundColor==rgb0))
					b.style.backgroundColor=col1;
			}
		}
	}
	// cols
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			a[j] = 0;
		}
		ok = 1;
		for (j=1;j<=9;j++) {
			k = sudoku[j][i];
			if(a[k]==0) a[k]=k; else ok=0;
		}
		if(ok==1){
			for (j=1;j<=9;j++) {
				b = document.getElementById('box'+j+i);
				if((b.style.backgroundColor==col1)||(b.style.backgroundColor==rgb1))
					b.style.backgroundColor=col2;
				if((b.style.backgroundColor==col0)||(b.style.backgroundColor==rgb0))
					b.style.backgroundColor=col1;
			}
		}
	}
	// squares
	for (i=1;i<=9;i++) {
		for (j=1;j<=9;j++) {
			a[j] = 0;
		}
		ok = 1;
		for (j=1;j<=9;j++) {
			ii = Math.floor(squares[i][j]/10);
			jj = squares[i][j]%10;
			k = sudoku[ii][jj];
			if(a[k]==0) a[k]=k; else ok=0;
		}
		if(ok==1){
			for (j=1;j<=9;j++) {
				ii = Math.floor(squares[i][j]/10);
				jj = squares[i][j]%10;
				b = document.getElementById('box'+ii+jj);
				if((b.style.backgroundColor==col2)||(b.style.backgroundColor==rgb2)) {
					b.style.backgroundColor=col3;
					c += 1;
				}
				if((b.style.backgroundColor==col1)||(b.style.backgroundColor==rgb1))
					b.style.backgroundColor=col2;
				if((b.style.backgroundColor==col0)||(b.style.backgroundColor==rgb0))
					b.style.backgroundColor=col1;
			}
		}
	}
	if(c==81){
		return 1; //sudoku is solved :)
	} else {
		return 0;
	}
}

function init_statics() {
	var i, j;
	for(i=1;i<=9;i++) {
		for(j=1;j<=9;j++) {
			statics[i][j] = 0;
		}
	}
}

function fill_statics(c) {
	var i, j, k=0;
	while(k<c) {
		i = Math.floor(Math.random()*10)%9+1;
		j = Math.floor(Math.random()*10)%9+1;
		if(statics[i][j]==0) {
			statics[i][j] = 1;
			k++;
		}
	}
}